use std::path::PathBuf;

use bevy::prelude::*;
use clap::Subcommand;

#[derive(Event, Debug, Subcommand, PartialEq, Eq)]
pub enum DebugEvent {
    /// Close the debug console.
    #[command(name = "close", aliases = ["close-console"])]
    CloseDebugConsole,

    /// Output a string to the console.
    ///
    /// Needed for logging in startup scripts, since the console isn't initialized yet.
    #[command(name = "echo", aliases = ["print", "print-to-console"])]
    PrintToConsole {
        /// Print as error.
        #[arg(short, long)]
        error: bool,
        /// The text to be printed in the console.
        text: String,
    },

    /// Log as INFO or ERROR to stdout.
    Log {
        /// Log as error.
        #[arg(short, long)]
        error: bool,
        /// The text to be logged.
        text: String,
    },

    /// Start the game
    StartGame,

    /// Run a tx file (the games debug scripting lang :3)
    #[command(name = "run")]
    RunFile { file: PathBuf },

    /// Enable debug mode.
    /// Will cause the game to behave differently (mostly, not do things by default it otherwise would.)
    ///
    /// Do note while all of this is part of the debug tooling, debug mode just gives the debug tools more controls of the game.
    ///
    /// Read: In debug mode, a lot of features of the normal game aren't enabled, and might have to be turned on explicitly.
    /// This is to allow for easier testing. The console etc. will still stay available regardless, just might complain that debug mode isn't enabled on some commands.
    ///
    /// For best results, enable debug mode before starting the game.
    ///
    /// Debug mode cannot be disabled. Just restart the game lmao.
    ///
    /// Current debug mode behaviour:
    /// - You can load gtlf scenes
    #[command(name = "enable-debug", aliases = ["debug-enable"])]
    EnableDebugMode,

    /// Load a gltf scene.
    ///
    /// Auto-reloading isn't possible due to engine reasons, see:
    /// https://github.com/bevyengine/bevy/pull/18358
    #[command(name = "load-scene", aliases = ["load-file", "load"])]
    LoadGltf {
        /// The asset path of the item.
        asset_path: String,
        /// Id for the item, so it can easily be selected using other debug tools.
        id: Option<String>,
    },

    /// Despawn an entity by string id. Uniqueness of Ids isn't guaranteed.
    Despawn { id: String },
}
